# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"test_fail": "00_test.ipynb",
         "test": "00_test.ipynb",
         "nequals": "00_test.ipynb",
         "test_eq": "00_test.ipynb",
         "test_eq_type": "00_test.ipynb",
         "test_ne": "00_test.ipynb",
         "is_close": "00_test.ipynb",
         "test_close": "00_test.ipynb",
         "test_is": "00_test.ipynb",
         "test_shuffled": "00_test.ipynb",
         "test_stdout": "00_test.ipynb",
         "test_warns": "00_test.ipynb",
         "TEST_IMAGE": "00_test.ipynb",
         "TEST_IMAGE_BW": "00_test.ipynb",
         "test_fig_exists": "00_test.ipynb",
         "ExceptionExpected": "00_test.ipynb",
         "exception": "00_test.ipynb",
         "defaults": "01_basics.ipynb",
         "ifnone": "01_basics.ipynb",
         "maybe_attr": "01_basics.ipynb",
         "basic_repr": "01_basics.ipynb",
         "is_array": "01_basics.ipynb",
         "listify": "01_basics.ipynb",
         "tuplify": "01_basics.ipynb",
         "true": "01_basics.ipynb",
         "NullType": "01_basics.ipynb",
         "null": "01_basics.ipynb",
         "tonull": "01_basics.ipynb",
         "get_class": "01_basics.ipynb",
         "mk_class": "01_basics.ipynb",
         "wrap_class": "01_basics.ipynb",
         "ignore_exceptions": "01_basics.ipynb",
         "exec_local": "01_basics.ipynb",
         "risinstance": "01_basics.ipynb",
         "Inf": "01_basics.ipynb",
         "in_": "01_basics.ipynb",
         "operator.in_": "01_basics.ipynb",
         "stop": "01_basics.ipynb",
         "gen": "01_basics.ipynb",
         "chunked": "01_basics.ipynb",
         "otherwise": "01_basics.ipynb",
         "custom_dir": "01_basics.ipynb",
         "AttrDict": "01_basics.ipynb",
         "get_annotations_ex": "01_basics.ipynb",
         "eval_type": "01_basics.ipynb",
         "type_hints": "01_basics.ipynb",
         "annotations": "01_basics.ipynb",
         "anno_ret": "01_basics.ipynb",
         "union2tuple": "01_basics.ipynb",
         "argnames": "01_basics.ipynb",
         "with_cast": "01_basics.ipynb",
         "store_attr": "01_basics.ipynb",
         "attrdict": "01_basics.ipynb",
         "properties": "01_basics.ipynb",
         "camel2words": "01_basics.ipynb",
         "camel2snake": "01_basics.ipynb",
         "snake2camel": "01_basics.ipynb",
         "class2attr": "01_basics.ipynb",
         "getcallable": "01_basics.ipynb",
         "getattrs": "01_basics.ipynb",
         "hasattrs": "01_basics.ipynb",
         "setattrs": "01_basics.ipynb",
         "try_attrs": "01_basics.ipynb",
         "GetAttrBase": "01_basics.ipynb",
         "GetAttr": "01_basics.ipynb",
         "delegate_attr": "01_basics.ipynb",
         "ShowPrint": "01_basics.ipynb",
         "Int": "01_basics.ipynb",
         "Str": "01_basics.ipynb",
         "Float": "01_basics.ipynb",
         "flatten": "01_basics.ipynb",
         "concat": "01_basics.ipynb",
         "strcat": "01_basics.ipynb",
         "detuplify": "01_basics.ipynb",
         "replicate": "01_basics.ipynb",
         "setify": "01_basics.ipynb",
         "merge": "01_basics.ipynb",
         "range_of": "01_basics.ipynb",
         "groupby": "01_basics.ipynb",
         "last_index": "01_basics.ipynb",
         "filter_dict": "01_basics.ipynb",
         "filter_keys": "01_basics.ipynb",
         "filter_values": "01_basics.ipynb",
         "cycle": "02_foundation.ipynb",
         "zip_cycle": "02_foundation.ipynb",
         "sorted_ex": "01_basics.ipynb",
         "not_": "01_basics.ipynb",
         "argwhere": "01_basics.ipynb",
         "filter_ex": "01_basics.ipynb",
         "renumerate": "01_basics.ipynb",
         "first": "01_basics.ipynb",
         "nested_attr": "01_basics.ipynb",
         "nested_callable": "01_basics.ipynb",
         "nested_idx": "01_basics.ipynb",
         "set_nested_idx": "01_basics.ipynb",
         "val2idx": "01_basics.ipynb",
         "uniqueify": "01_basics.ipynb",
         "loop_first_last": "01_basics.ipynb",
         "loop_first": "01_basics.ipynb",
         "loop_last": "01_basics.ipynb",
         "num_methods": "01_basics.ipynb",
         "rnum_methods": "01_basics.ipynb",
         "inum_methods": "01_basics.ipynb",
         "fastuple": "01_basics.ipynb",
         "arg0": "01_basics.ipynb",
         "arg1": "01_basics.ipynb",
         "arg2": "01_basics.ipynb",
         "arg3": "01_basics.ipynb",
         "arg4": "01_basics.ipynb",
         "bind": "01_basics.ipynb",
         "mapt": "01_basics.ipynb",
         "map_ex": "01_basics.ipynb",
         "compose": "01_basics.ipynb",
         "maps": "01_basics.ipynb",
         "partialler": "01_basics.ipynb",
         "instantiate": "01_basics.ipynb",
         "using_attr": "01_basics.ipynb",
         "Self": "01_basics.ipynb",
         "copy_func": "01_basics.ipynb",
         "patch_to": "01_basics.ipynb",
         "patch": "01_basics.ipynb",
         "patch_property": "01_basics.ipynb",
         "compile_re": "01_basics.ipynb",
         "ImportEnum": "01_basics.ipynb",
         "StrEnum": "01_basics.ipynb",
         "str_enum": "01_basics.ipynb",
         "Stateful": "01_basics.ipynb",
         "PrettyString": "01_basics.ipynb",
         "even_mults": "01_basics.ipynb",
         "num_cpus": "01_basics.ipynb",
         "defaults.cpus": "01_basics.ipynb",
         "add_props": "01_basics.ipynb",
         "typed": "01_basics.ipynb",
         "exec_new": "01_basics.ipynb",
         "exec_import": "01_basics.ipynb",
         "working_directory": "02_foundation.ipynb",
         "add_docs": "02_foundation.ipynb",
         "docs": "02_foundation.ipynb",
         "coll_repr": "02_foundation.ipynb",
         "is_bool": "02_foundation.ipynb",
         "mask2idxs": "02_foundation.ipynb",
         "is_indexer": "02_foundation.ipynb",
         "CollBase": "02_foundation.ipynb",
         "L": "02_foundation.ipynb",
         "L.__signature__": "02_foundation.ipynb",
         "save_config_file": "02_foundation.ipynb",
         "read_config_file": "02_foundation.ipynb",
         "Config": "02_foundation.ipynb",
         "walk": "03_xtras.ipynb",
         "globtastic": "03_xtras.ipynb",
         "maybe_open": "03_xtras.ipynb",
         "image_size": "03_xtras.ipynb",
         "bunzip": "03_xtras.ipynb",
         "loads": "03_xtras.ipynb",
         "loads_multi": "03_xtras.ipynb",
         "dumps": "03_xtras.ipynb",
         "untar_dir": "03_xtras.ipynb",
         "repo_details": "03_xtras.ipynb",
         "run": "03_xtras.ipynb",
         "open_file": "03_xtras.ipynb",
         "save_pickle": "03_xtras.ipynb",
         "load_pickle": "03_xtras.ipynb",
         "dict2obj": "03_xtras.ipynb",
         "obj2dict": "03_xtras.ipynb",
         "repr_dict": "03_xtras.ipynb",
         "is_listy": "03_xtras.ipynb",
         "mapped": "03_xtras.ipynb",
         "Path.readlines": "03_xtras.ipynb",
         "Path.read_json": "03_xtras.ipynb",
         "Path.mk_write": "03_xtras.ipynb",
         "Path.relpath": "03_xtras.ipynb",
         "Path.ls": "03_xtras.ipynb",
         "Path.__repr__": "03_xtras.ipynb",
         "Path.delete": "03_xtras.ipynb",
         "IterLen": "03_xtras.ipynb",
         "ReindexCollection": "03_xtras.ipynb",
         "get_source_link": "03_xtras.ipynb",
         "truncstr": "03_xtras.ipynb",
         "spark_chars": "03_xtras.ipynb",
         "sparkline": "03_xtras.ipynb",
         "modify_exception": "03_xtras.ipynb",
         "round_multiple": "03_xtras.ipynb",
         "str2bool": "03_xtras.ipynb",
         "set_num_threads": "03_xtras.ipynb",
         "join_path_file": "03_xtras.ipynb",
         "autostart": "03_xtras.ipynb",
         "EventTimer": "03_xtras.ipynb",
         "stringfmt_names": "03_xtras.ipynb",
         "PartialFormatter": "03_xtras.ipynb",
         "partial_format": "03_xtras.ipynb",
         "utc2local": "03_xtras.ipynb",
         "local2utc": "03_xtras.ipynb",
         "trace": "03_xtras.ipynb",
         "modified_env": "03_xtras.ipynb",
         "ContextManagers": "03_xtras.ipynb",
         "shufflish": "03_xtras.ipynb",
         "threaded": "03a_parallel.ipynb",
         "startthread": "03a_parallel.ipynb",
         "parallelable": "03a_parallel.ipynb",
         "ThreadPoolExecutor": "03a_parallel.ipynb",
         "ProcessPoolExecutor": "03a_parallel.ipynb",
         "parallel": "03a_parallel.ipynb",
         "add_one": "03a_parallel.ipynb",
         "run_procs": "03a_parallel.ipynb",
         "parallel_gen": "03a_parallel.ipynb",
         "url_default_headers": "03b_net.ipynb",
         "urlquote": "03b_net.ipynb",
         "urlwrap": "03b_net.ipynb",
         "ExceptionsHTTP": "03b_net.ipynb",
         "HTTP4xxClientError": "03b_net.ipynb",
         "HTTP5xxServerError": "03b_net.ipynb",
         "urlopen": "03b_net.ipynb",
         "urlread": "03b_net.ipynb",
         "urljson": "03b_net.ipynb",
         "urlcheck": "03b_net.ipynb",
         "urlclean": "03b_net.ipynb",
         "urlretrieve": "03b_net.ipynb",
         "urldest": "03b_net.ipynb",
         "urlsave": "03b_net.ipynb",
         "urlvalid": "03b_net.ipynb",
         "urlrequest": "03b_net.ipynb",
         "Request.summary": "03b_net.ipynb",
         "urlsend": "03b_net.ipynb",
         "do_request": "03b_net.ipynb",
         "start_server": "03b_net.ipynb",
         "start_client": "03b_net.ipynb",
         "lenient_issubclass": "04_dispatch.ipynb",
         "sorted_topologically": "04_dispatch.ipynb",
         "TypeDispatch": "04_dispatch.ipynb",
         "DispatchReg": "04_dispatch.ipynb",
         "typedispatch": "04_dispatch.ipynb",
         "retain_meta": "04_dispatch.ipynb",
         "default_set_meta": "04_dispatch.ipynb",
         "retain_type": "04_dispatch.ipynb",
         "retain_types": "04_dispatch.ipynb",
         "explode_types": "04_dispatch.ipynb",
         "Transform": "05_transform.ipynb",
         "InplaceTransform": "05_transform.ipynb",
         "DisplayedTransform": "05_transform.ipynb",
         "ItemTransform": "05_transform.ipynb",
         "get_func": "05_transform.ipynb",
         "Func": "05_transform.ipynb",
         "Sig": "05_transform.ipynb",
         "compose_tfms": "05_transform.ipynb",
         "mk_transform": "05_transform.ipynb",
         "gather_attrs": "05_transform.ipynb",
         "gather_attr_names": "05_transform.ipynb",
         "Pipeline": "05_transform.ipynb",
         "docstring": "06_docments.ipynb",
         "parse_docstring": "06_docments.ipynb",
         "isdataclass": "06_docments.ipynb",
         "get_dataclass_source": "06_docments.ipynb",
         "get_source": "06_docments.ipynb",
         "empty": "06_docments.ipynb",
         "get_name": "06_docments.ipynb",
         "qual_name": "06_docments.ipynb",
         "docments": "06_docments.ipynb",
         "test_sig": "07_meta.ipynb",
         "FixSigMeta": "07_meta.ipynb",
         "PrePostInitMeta": "07_meta.ipynb",
         "AutoInit": "07_meta.ipynb",
         "NewChkMeta": "07_meta.ipynb",
         "BypassNewMeta": "07_meta.ipynb",
         "empty2none": "07_meta.ipynb",
         "anno_dict": "07_meta.ipynb",
         "use_kwargs_dict": "07_meta.ipynb",
         "use_kwargs": "07_meta.ipynb",
         "delegates": "07_meta.ipynb",
         "method": "07_meta.ipynb",
         "funcs_kwargs": "07_meta.ipynb",
         "store_true": "08_script.ipynb",
         "store_false": "08_script.ipynb",
         "bool_arg": "08_script.ipynb",
         "clean_type_str": "08_script.ipynb",
         "Param": "08_script.ipynb",
         "anno_parser": "08_script.ipynb",
         "args_from_prog": "08_script.ipynb",
         "SCRIPT_INFO": "08_script.ipynb",
         "call_parse": "08_script.ipynb"}

modules = ["test.py",
           "basics.py",
           "foundation.py",
           "xtras.py",
           "parallel.py",
           "net.py",
           "dispatch.py",
           "transform.py",
           "docments.py",
           "meta.py",
           "script.py"]

doc_url = "https://fastcore.fast.ai/"

git_url = "https://github.com/fastai/fastcore/tree/master/"

def custom_doc_links(name):
    from nbdev.showdoc import try_external_doc_link
    return try_external_doc_link(name, ['fastai', 'nbdev'])



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/',
                'doc_host': 'https://fastcore.fast.ai',
                'git_url': 'https://github.com/fastai/fastcore/',
                'lib_path': 'fastcore'},
  'syms': { 'fastcore.all': {},
            'fastcore.basics': { 'fastcore.basics.AttrDict': ('basics.html#attrdict', 'fastcore/basics.py'),
                                 'fastcore.basics.AttrDict.__dir__': ('basics.html#attrdict.__dir__', 'fastcore/basics.py'),
                                 'fastcore.basics.AttrDict.__getattr__': ('basics.html#attrdict.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics.AttrDict.__setattr__': ('basics.html#attrdict.__setattr__', 'fastcore/basics.py'),
                                 'fastcore.basics.AttrDict._repr_markdown_': ('basics.html#attrdict._repr_markdown_', 'fastcore/basics.py'),
                                 'fastcore.basics.AttrDict.copy': ('basics.html#attrdict.copy', 'fastcore/basics.py'),
                                 'fastcore.basics.Float': ('basics.html#float', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr': ('basics.html#getattr', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr.__dir__': ('basics.html#getattr.__dir__', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr.__getattr__': ('basics.html#getattr.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr.__setstate__': ('basics.html#getattr.__setstate__', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr._component_attr_filter': ( 'basics.html#getattr._component_attr_filter',
                                                                                     'fastcore/basics.py'),
                                 'fastcore.basics.GetAttr._dir': ('basics.html#getattr._dir', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttrBase': ('basics.html#getattrbase', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttrBase.__dir__': ('basics.html#getattrbase.__dir__', 'fastcore/basics.py'),
                                 'fastcore.basics.GetAttrBase.__getattr__': ('basics.html#getattrbase.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics.ImportEnum': ('basics.html#importenum', 'fastcore/basics.py'),
                                 'fastcore.basics.ImportEnum.imports': ('basics.html#importenum.imports', 'fastcore/basics.py'),
                                 'fastcore.basics.Inf': ('basics.html#inf', 'fastcore/basics.py'),
                                 'fastcore.basics.Int': ('basics.html#int', 'fastcore/basics.py'),
                                 'fastcore.basics.NullType': ('basics.html#nulltype', 'fastcore/basics.py'),
                                 'fastcore.basics.NullType.__bool__': ('basics.html#nulltype.__bool__', 'fastcore/basics.py'),
                                 'fastcore.basics.NullType.__call__': ('basics.html#nulltype.__call__', 'fastcore/basics.py'),
                                 'fastcore.basics.NullType.__getattr__': ('basics.html#nulltype.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics.NullType.__getitem__': ('basics.html#nulltype.__getitem__', 'fastcore/basics.py'),
                                 'fastcore.basics.PrettyString': ('basics.html#prettystring', 'fastcore/basics.py'),
                                 'fastcore.basics.PrettyString.__repr__': ('basics.html#prettystring.__repr__', 'fastcore/basics.py'),
                                 'fastcore.basics.ShowPrint': ('basics.html#showprint', 'fastcore/basics.py'),
                                 'fastcore.basics.ShowPrint.show': ('basics.html#showprint.show', 'fastcore/basics.py'),
                                 'fastcore.basics.Stateful': ('basics.html#stateful', 'fastcore/basics.py'),
                                 'fastcore.basics.Stateful.__getstate__': ('basics.html#stateful.__getstate__', 'fastcore/basics.py'),
                                 'fastcore.basics.Stateful.__init__': ('basics.html#stateful.__init__', 'fastcore/basics.py'),
                                 'fastcore.basics.Stateful.__setstate__': ('basics.html#stateful.__setstate__', 'fastcore/basics.py'),
                                 'fastcore.basics.Stateful._init_state': ('basics.html#stateful._init_state', 'fastcore/basics.py'),
                                 'fastcore.basics.Str': ('basics.html#str', 'fastcore/basics.py'),
                                 'fastcore.basics.StrEnum': ('basics.html#strenum', 'fastcore/basics.py'),
                                 'fastcore.basics.StrEnum.__str__': ('basics.html#strenum.__str__', 'fastcore/basics.py'),
                                 'fastcore.basics._Arg': ('basics.html#_arg', 'fastcore/basics.py'),
                                 'fastcore.basics._Arg.__init__': ('basics.html#_arg.__init__', 'fastcore/basics.py'),
                                 'fastcore.basics._InfMeta': ('basics.html#_infmeta', 'fastcore/basics.py'),
                                 'fastcore.basics._InfMeta.count': ('basics.html#_infmeta.count', 'fastcore/basics.py'),
                                 'fastcore.basics._InfMeta.nones': ('basics.html#_infmeta.nones', 'fastcore/basics.py'),
                                 'fastcore.basics._InfMeta.ones': ('basics.html#_infmeta.ones', 'fastcore/basics.py'),
                                 'fastcore.basics._InfMeta.zeros': ('basics.html#_infmeta.zeros', 'fastcore/basics.py'),
                                 'fastcore.basics._Self': ('basics.html#_self', 'fastcore/basics.py'),
                                 'fastcore.basics._Self.__call__': ('basics.html#_self.__call__', 'fastcore/basics.py'),
                                 'fastcore.basics._Self.__getattr__': ('basics.html#_self.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics._Self.__init__': ('basics.html#_self.__init__', 'fastcore/basics.py'),
                                 'fastcore.basics._Self.__repr__': ('basics.html#_self.__repr__', 'fastcore/basics.py'),
                                 'fastcore.basics._Self._call': ('basics.html#_self._call', 'fastcore/basics.py'),
                                 'fastcore.basics._SelfCls': ('basics.html#_selfcls', 'fastcore/basics.py'),
                                 'fastcore.basics._SelfCls.__call__': ('basics.html#_selfcls.__call__', 'fastcore/basics.py'),
                                 'fastcore.basics._SelfCls.__getattr__': ('basics.html#_selfcls.__getattr__', 'fastcore/basics.py'),
                                 'fastcore.basics._SelfCls.__getitem__': ('basics.html#_selfcls.__getitem__', 'fastcore/basics.py'),
                                 'fastcore.basics._access': ('basics.html#_access', 'fastcore/basics.py'),
                                 'fastcore.basics._eval_type': ('basics.html#_eval_type', 'fastcore/basics.py'),
                                 'fastcore.basics._get_op': ('basics.html#_get_op', 'fastcore/basics.py'),
                                 'fastcore.basics._ispy3_10': ('basics.html#_ispy3_10', 'fastcore/basics.py'),
                                 'fastcore.basics._mk_op': ('basics.html#_mk_op', 'fastcore/basics.py'),
                                 'fastcore.basics._nested_idx': ('basics.html#_nested_idx', 'fastcore/basics.py'),
                                 'fastcore.basics._oper': ('basics.html#_oper', 'fastcore/basics.py'),
                                 'fastcore.basics._store_attr': ('basics.html#_store_attr', 'fastcore/basics.py'),
                                 'fastcore.basics._typeerr': ('basics.html#_typeerr', 'fastcore/basics.py'),
                                 'fastcore.basics._using_attr': ('basics.html#_using_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.add_props': ('basics.html#add_props', 'fastcore/basics.py'),
                                 'fastcore.basics.anno_ret': ('basics.html#anno_ret', 'fastcore/basics.py'),
                                 'fastcore.basics.annotations': ('basics.html#annotations', 'fastcore/basics.py'),
                                 'fastcore.basics.argnames': ('basics.html#argnames', 'fastcore/basics.py'),
                                 'fastcore.basics.argwhere': ('basics.html#argwhere', 'fastcore/basics.py'),
                                 'fastcore.basics.attrdict': ('basics.html#attrdict', 'fastcore/basics.py'),
                                 'fastcore.basics.basic_repr': ('basics.html#basic_repr', 'fastcore/basics.py'),
                                 'fastcore.basics.bind': ('basics.html#bind', 'fastcore/basics.py'),
                                 'fastcore.basics.bind.__call__': ('basics.html#bind.__call__', 'fastcore/basics.py'),
                                 'fastcore.basics.bind.__init__': ('basics.html#bind.__init__', 'fastcore/basics.py'),
                                 'fastcore.basics.camel2snake': ('basics.html#camel2snake', 'fastcore/basics.py'),
                                 'fastcore.basics.camel2words': ('basics.html#camel2words', 'fastcore/basics.py'),
                                 'fastcore.basics.chunked': ('basics.html#chunked', 'fastcore/basics.py'),
                                 'fastcore.basics.class2attr': ('basics.html#class2attr', 'fastcore/basics.py'),
                                 'fastcore.basics.compile_re': ('basics.html#compile_re', 'fastcore/basics.py'),
                                 'fastcore.basics.compose': ('basics.html#compose', 'fastcore/basics.py'),
                                 'fastcore.basics.concat': ('basics.html#concat', 'fastcore/basics.py'),
                                 'fastcore.basics.copy_func': ('basics.html#copy_func', 'fastcore/basics.py'),
                                 'fastcore.basics.custom_dir': ('basics.html#custom_dir', 'fastcore/basics.py'),
                                 'fastcore.basics.cycle': ('basics.html#cycle', 'fastcore/basics.py'),
                                 'fastcore.basics.delegate_attr': ('basics.html#delegate_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.detuplify': ('basics.html#detuplify', 'fastcore/basics.py'),
                                 'fastcore.basics.eval_type': ('basics.html#eval_type', 'fastcore/basics.py'),
                                 'fastcore.basics.even_mults': ('basics.html#even_mults', 'fastcore/basics.py'),
                                 'fastcore.basics.exec_import': ('basics.html#exec_import', 'fastcore/basics.py'),
                                 'fastcore.basics.exec_local': ('basics.html#exec_local', 'fastcore/basics.py'),
                                 'fastcore.basics.exec_new': ('basics.html#exec_new', 'fastcore/basics.py'),
                                 'fastcore.basics.fastuple': ('basics.html#fastuple', 'fastcore/basics.py'),
                                 'fastcore.basics.fastuple.__new__': ('basics.html#fastuple.__new__', 'fastcore/basics.py'),
                                 'fastcore.basics.fastuple._op': ('basics.html#fastuple._op', 'fastcore/basics.py'),
                                 'fastcore.basics.fastuple.add': ('basics.html#fastuple.add', 'fastcore/basics.py'),
                                 'fastcore.basics.fastuple.mul': ('basics.html#fastuple.mul', 'fastcore/basics.py'),
                                 'fastcore.basics.filter_dict': ('basics.html#filter_dict', 'fastcore/basics.py'),
                                 'fastcore.basics.filter_ex': ('basics.html#filter_ex', 'fastcore/basics.py'),
                                 'fastcore.basics.filter_keys': ('basics.html#filter_keys', 'fastcore/basics.py'),
                                 'fastcore.basics.filter_values': ('basics.html#filter_values', 'fastcore/basics.py'),
                                 'fastcore.basics.first': ('basics.html#first', 'fastcore/basics.py'),
                                 'fastcore.basics.flatten': ('basics.html#flatten', 'fastcore/basics.py'),
                                 'fastcore.basics.gen': ('basics.html#gen', 'fastcore/basics.py'),
                                 'fastcore.basics.get_annotations_ex': ('basics.html#get_annotations_ex', 'fastcore/basics.py'),
                                 'fastcore.basics.get_class': ('basics.html#get_class', 'fastcore/basics.py'),
                                 'fastcore.basics.getattrs': ('basics.html#getattrs', 'fastcore/basics.py'),
                                 'fastcore.basics.getcallable': ('basics.html#getcallable', 'fastcore/basics.py'),
                                 'fastcore.basics.groupby': ('basics.html#groupby', 'fastcore/basics.py'),
                                 'fastcore.basics.hasattrs': ('basics.html#hasattrs', 'fastcore/basics.py'),
                                 'fastcore.basics.ifnone': ('basics.html#ifnone', 'fastcore/basics.py'),
                                 'fastcore.basics.ignore_exceptions': ('basics.html#ignore_exceptions', 'fastcore/basics.py'),
                                 'fastcore.basics.ignore_exceptions.__enter__': ( 'basics.html#ignore_exceptions.__enter__',
                                                                                  'fastcore/basics.py'),
                                 'fastcore.basics.ignore_exceptions.__exit__': ( 'basics.html#ignore_exceptions.__exit__',
                                                                                 'fastcore/basics.py'),
                                 'fastcore.basics.in_': ('basics.html#in_', 'fastcore/basics.py'),
                                 'fastcore.basics.instantiate': ('basics.html#instantiate', 'fastcore/basics.py'),
                                 'fastcore.basics.is_array': ('basics.html#is_array', 'fastcore/basics.py'),
                                 'fastcore.basics.last_index': ('basics.html#last_index', 'fastcore/basics.py'),
                                 'fastcore.basics.listify': ('basics.html#listify', 'fastcore/basics.py'),
                                 'fastcore.basics.loop_first': ('basics.html#loop_first', 'fastcore/basics.py'),
                                 'fastcore.basics.loop_first_last': ('basics.html#loop_first_last', 'fastcore/basics.py'),
                                 'fastcore.basics.loop_last': ('basics.html#loop_last', 'fastcore/basics.py'),
                                 'fastcore.basics.map_ex': ('basics.html#map_ex', 'fastcore/basics.py'),
                                 'fastcore.basics.maps': ('basics.html#maps', 'fastcore/basics.py'),
                                 'fastcore.basics.mapt': ('basics.html#mapt', 'fastcore/basics.py'),
                                 'fastcore.basics.maybe_attr': ('basics.html#maybe_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.merge': ('basics.html#merge', 'fastcore/basics.py'),
                                 'fastcore.basics.mk_class': ('basics.html#mk_class', 'fastcore/basics.py'),
                                 'fastcore.basics.nested_attr': ('basics.html#nested_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.nested_callable': ('basics.html#nested_callable', 'fastcore/basics.py'),
                                 'fastcore.basics.nested_idx': ('basics.html#nested_idx', 'fastcore/basics.py'),
                                 'fastcore.basics.nested_setdefault': ('basics.html#nested_setdefault', 'fastcore/basics.py'),
                                 'fastcore.basics.not_': ('basics.html#not_', 'fastcore/basics.py'),
                                 'fastcore.basics.num_cpus': ('basics.html#num_cpus', 'fastcore/basics.py'),
                                 'fastcore.basics.only': ('basics.html#only', 'fastcore/basics.py'),
                                 'fastcore.basics.otherwise': ('basics.html#otherwise', 'fastcore/basics.py'),
                                 'fastcore.basics.partialler': ('basics.html#partialler', 'fastcore/basics.py'),
                                 'fastcore.basics.patch': ('basics.html#patch', 'fastcore/basics.py'),
                                 'fastcore.basics.patch_property': ('basics.html#patch_property', 'fastcore/basics.py'),
                                 'fastcore.basics.patch_to': ('basics.html#patch_to', 'fastcore/basics.py'),
                                 'fastcore.basics.properties': ('basics.html#properties', 'fastcore/basics.py'),
                                 'fastcore.basics.range_of': ('basics.html#range_of', 'fastcore/basics.py'),
                                 'fastcore.basics.renumerate': ('basics.html#renumerate', 'fastcore/basics.py'),
                                 'fastcore.basics.replicate': ('basics.html#replicate', 'fastcore/basics.py'),
                                 'fastcore.basics.ret_false': ('basics.html#ret_false', 'fastcore/basics.py'),
                                 'fastcore.basics.ret_true': ('basics.html#ret_true', 'fastcore/basics.py'),
                                 'fastcore.basics.risinstance': ('basics.html#risinstance', 'fastcore/basics.py'),
                                 'fastcore.basics.set_nested_idx': ('basics.html#set_nested_idx', 'fastcore/basics.py'),
                                 'fastcore.basics.setattrs': ('basics.html#setattrs', 'fastcore/basics.py'),
                                 'fastcore.basics.setify': ('basics.html#setify', 'fastcore/basics.py'),
                                 'fastcore.basics.signature_ex': ('basics.html#signature_ex', 'fastcore/basics.py'),
                                 'fastcore.basics.snake2camel': ('basics.html#snake2camel', 'fastcore/basics.py'),
                                 'fastcore.basics.sorted_ex': ('basics.html#sorted_ex', 'fastcore/basics.py'),
                                 'fastcore.basics.stop': ('basics.html#stop', 'fastcore/basics.py'),
                                 'fastcore.basics.store_attr': ('basics.html#store_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.str2bool': ('basics.html#str2bool', 'fastcore/basics.py'),
                                 'fastcore.basics.str_enum': ('basics.html#str_enum', 'fastcore/basics.py'),
                                 'fastcore.basics.strcat': ('basics.html#strcat', 'fastcore/basics.py'),
                                 'fastcore.basics.tonull': ('basics.html#tonull', 'fastcore/basics.py'),
                                 'fastcore.basics.true': ('basics.html#true', 'fastcore/basics.py'),
                                 'fastcore.basics.try_attrs': ('basics.html#try_attrs', 'fastcore/basics.py'),
                                 'fastcore.basics.tuplify': ('basics.html#tuplify', 'fastcore/basics.py'),
                                 'fastcore.basics.type_hints': ('basics.html#type_hints', 'fastcore/basics.py'),
                                 'fastcore.basics.typed': ('basics.html#typed', 'fastcore/basics.py'),
                                 'fastcore.basics.union2tuple': ('basics.html#union2tuple', 'fastcore/basics.py'),
                                 'fastcore.basics.uniqueify': ('basics.html#uniqueify', 'fastcore/basics.py'),
                                 'fastcore.basics.using_attr': ('basics.html#using_attr', 'fastcore/basics.py'),
                                 'fastcore.basics.val2idx': ('basics.html#val2idx', 'fastcore/basics.py'),
                                 'fastcore.basics.with_cast': ('basics.html#with_cast', 'fastcore/basics.py'),
                                 'fastcore.basics.wrap_class': ('basics.html#wrap_class', 'fastcore/basics.py'),
                                 'fastcore.basics.zip_cycle': ('basics.html#zip_cycle', 'fastcore/basics.py')},
            'fastcore.dispatch': { 'fastcore.dispatch.DispatchReg': ('dispatch.html#dispatchreg', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.DispatchReg.__call__': ('dispatch.html#dispatchreg.__call__', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.DispatchReg.__init__': ('dispatch.html#dispatchreg.__init__', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch': ('dispatch.html#typedispatch', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.__call__': ( 'dispatch.html#typedispatch.__call__',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.__get__': ('dispatch.html#typedispatch.__get__', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.__getitem__': ( 'dispatch.html#typedispatch.__getitem__',
                                                                                   'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.__init__': ( 'dispatch.html#typedispatch.__init__',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.__repr__': ( 'dispatch.html#typedispatch.__repr__',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch._attname': ( 'dispatch.html#typedispatch._attname',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.add': ('dispatch.html#typedispatch.add', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.first': ('dispatch.html#typedispatch.first', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.TypeDispatch.returns': ('dispatch.html#typedispatch.returns', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict': ('dispatch.html#_typedict', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.__getitem__': ( 'dispatch.html#_typedict.__getitem__',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.__init__': ('dispatch.html#_typedict.__init__', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.__repr__': ('dispatch.html#_typedict.__repr__', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict._reset': ('dispatch.html#_typedict._reset', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.add': ('dispatch.html#_typedict.add', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.all_matches': ( 'dispatch.html#_typedict.all_matches',
                                                                                'fastcore/dispatch.py'),
                                   'fastcore.dispatch._TypeDict.first': ('dispatch.html#_typedict.first', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._chk_defaults': ('dispatch.html#_chk_defaults', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch._p2_anno': ('dispatch.html#_p2_anno', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.cast': ('dispatch.html#cast', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.default_set_meta': ('dispatch.html#default_set_meta', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.explode_types': ('dispatch.html#explode_types', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.lenient_issubclass': ('dispatch.html#lenient_issubclass', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.retain_meta': ('dispatch.html#retain_meta', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.retain_type': ('dispatch.html#retain_type', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.retain_types': ('dispatch.html#retain_types', 'fastcore/dispatch.py'),
                                   'fastcore.dispatch.sorted_topologically': ( 'dispatch.html#sorted_topologically',
                                                                               'fastcore/dispatch.py')},
            'fastcore.docments': { 'fastcore.docments._clean_comment': ('docments.html#_clean_comment', 'fastcore/docments.py'),
                                   'fastcore.docments._docments': ('docments.html#_docments', 'fastcore/docments.py'),
                                   'fastcore.docments._get_comment': ('docments.html#_get_comment', 'fastcore/docments.py'),
                                   'fastcore.docments._get_full': ('docments.html#_get_full', 'fastcore/docments.py'),
                                   'fastcore.docments._get_property_name': ('docments.html#_get_property_name', 'fastcore/docments.py'),
                                   'fastcore.docments._merge_doc': ('docments.html#_merge_doc', 'fastcore/docments.py'),
                                   'fastcore.docments._merge_docs': ('docments.html#_merge_docs', 'fastcore/docments.py'),
                                   'fastcore.docments._param_locs': ('docments.html#_param_locs', 'fastcore/docments.py'),
                                   'fastcore.docments._parses': ('docments.html#_parses', 'fastcore/docments.py'),
                                   'fastcore.docments._tokens': ('docments.html#_tokens', 'fastcore/docments.py'),
                                   'fastcore.docments.docments': ('docments.html#docments', 'fastcore/docments.py'),
                                   'fastcore.docments.docstring': ('docments.html#docstring', 'fastcore/docments.py'),
                                   'fastcore.docments.get_dataclass_source': ('docments.html#get_dataclass_source', 'fastcore/docments.py'),
                                   'fastcore.docments.get_name': ('docments.html#get_name', 'fastcore/docments.py'),
                                   'fastcore.docments.get_source': ('docments.html#get_source', 'fastcore/docments.py'),
                                   'fastcore.docments.isdataclass': ('docments.html#isdataclass', 'fastcore/docments.py'),
                                   'fastcore.docments.parse_docstring': ('docments.html#parse_docstring', 'fastcore/docments.py'),
                                   'fastcore.docments.qual_name': ('docments.html#qual_name', 'fastcore/docments.py')},
            'fastcore.docscrape': {},
            'fastcore.foundation': { 'fastcore.foundation.CollBase': ('foundation.html#collbase', 'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__delitem__': ( 'foundation.html#collbase.__delitem__',
                                                                                   'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__getitem__': ( 'foundation.html#collbase.__getitem__',
                                                                                   'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__init__': ( 'foundation.html#collbase.__init__',
                                                                                'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__iter__': ( 'foundation.html#collbase.__iter__',
                                                                                'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__len__': ('foundation.html#collbase.__len__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__repr__': ( 'foundation.html#collbase.__repr__',
                                                                                'fastcore/foundation.py'),
                                     'fastcore.foundation.CollBase.__setitem__': ( 'foundation.html#collbase.__setitem__',
                                                                                   'fastcore/foundation.py'),
                                     'fastcore.foundation.Config': ('foundation.html#config', 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__contains__': ( 'foundation.html#config.__contains__',
                                                                                  'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__getattr__': ( 'foundation.html#config.__getattr__',
                                                                                 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__getitem__': ( 'foundation.html#config.__getitem__',
                                                                                 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__init__': ('foundation.html#config.__init__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__repr__': ('foundation.html#config.__repr__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.__setitem__': ( 'foundation.html#config.__setitem__',
                                                                                 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.get': ('foundation.html#config.get', 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.path': ('foundation.html#config.path', 'fastcore/foundation.py'),
                                     'fastcore.foundation.Config.save': ('foundation.html#config.save', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L': ('foundation.html#l', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__add__': ('foundation.html#l.__add__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__addi__': ('foundation.html#l.__addi__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__contains__': ('foundation.html#l.__contains__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__eq__': ('foundation.html#l.__eq__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__getitem__': ('foundation.html#l.__getitem__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__init__': ('foundation.html#l.__init__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__invert__': ('foundation.html#l.__invert__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__iter__': ('foundation.html#l.__iter__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__mul__': ('foundation.html#l.__mul__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__radd__': ('foundation.html#l.__radd__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__repr__': ('foundation.html#l.__repr__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__reversed__': ('foundation.html#l.__reversed__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.__setitem__': ('foundation.html#l.__setitem__', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L._get': ('foundation.html#l._get', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L._new': ('foundation.html#l._new', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L._repr_pretty_': ('foundation.html#l._repr_pretty_', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L._xtra': ('foundation.html#l._xtra', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.argfirst': ('foundation.html#l.argfirst', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.argwhere': ('foundation.html#l.argwhere', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.attrgot': ('foundation.html#l.attrgot', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.concat': ('foundation.html#l.concat', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.copy': ('foundation.html#l.copy', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.cycle': ('foundation.html#l.cycle', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.enumerate': ('foundation.html#l.enumerate', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.filter': ('foundation.html#l.filter', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.itemgot': ('foundation.html#l.itemgot', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.map': ('foundation.html#l.map', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.map_dict': ('foundation.html#l.map_dict', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.map_first': ('foundation.html#l.map_first', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.map_zip': ('foundation.html#l.map_zip', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.map_zipwith': ('foundation.html#l.map_zipwith', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.product': ('foundation.html#l.product', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.range': ('foundation.html#l.range', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.reduce': ('foundation.html#l.reduce', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.renumerate': ('foundation.html#l.renumerate', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.setattrs': ('foundation.html#l.setattrs', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.shuffle': ('foundation.html#l.shuffle', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.sorted': ('foundation.html#l.sorted', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.split': ('foundation.html#l.split', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.starmap': ('foundation.html#l.starmap', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.sum': ('foundation.html#l.sum', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.unique': ('foundation.html#l.unique', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.val2idx': ('foundation.html#l.val2idx', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.zip': ('foundation.html#l.zip', 'fastcore/foundation.py'),
                                     'fastcore.foundation.L.zipwith': ('foundation.html#l.zipwith', 'fastcore/foundation.py'),
                                     'fastcore.foundation._L_Meta': ('foundation.html#_l_meta', 'fastcore/foundation.py'),
                                     'fastcore.foundation._L_Meta.__call__': ('foundation.html#_l_meta.__call__', 'fastcore/foundation.py'),
                                     'fastcore.foundation._f': ('foundation.html#_f', 'fastcore/foundation.py'),
                                     'fastcore.foundation.add_docs': ('foundation.html#add_docs', 'fastcore/foundation.py'),
                                     'fastcore.foundation.coll_repr': ('foundation.html#coll_repr', 'fastcore/foundation.py'),
                                     'fastcore.foundation.cycle': ('foundation.html#cycle', 'fastcore/foundation.py'),
                                     'fastcore.foundation.docs': ('foundation.html#docs', 'fastcore/foundation.py'),
                                     'fastcore.foundation.is_bool': ('foundation.html#is_bool', 'fastcore/foundation.py'),
                                     'fastcore.foundation.is_indexer': ('foundation.html#is_indexer', 'fastcore/foundation.py'),
                                     'fastcore.foundation.mask2idxs': ('foundation.html#mask2idxs', 'fastcore/foundation.py'),
                                     'fastcore.foundation.read_config_file': ('foundation.html#read_config_file', 'fastcore/foundation.py'),
                                     'fastcore.foundation.save_config_file': ('foundation.html#save_config_file', 'fastcore/foundation.py'),
                                     'fastcore.foundation.working_directory': ( 'foundation.html#working_directory',
                                                                                'fastcore/foundation.py'),
                                     'fastcore.foundation.zip_cycle': ('foundation.html#zip_cycle', 'fastcore/foundation.py')},
            'fastcore.imports': {},
            'fastcore.meta': { 'fastcore.meta.AutoInit': ('meta.html#autoinit', 'fastcore/meta.py'),
                               'fastcore.meta.AutoInit.__pre_init__': ('meta.html#autoinit.__pre_init__', 'fastcore/meta.py'),
                               'fastcore.meta.BypassNewMeta': ('meta.html#bypassnewmeta', 'fastcore/meta.py'),
                               'fastcore.meta.BypassNewMeta.__call__': ('meta.html#bypassnewmeta.__call__', 'fastcore/meta.py'),
                               'fastcore.meta.FixSigMeta': ('meta.html#fixsigmeta', 'fastcore/meta.py'),
                               'fastcore.meta.FixSigMeta.__new__': ('meta.html#fixsigmeta.__new__', 'fastcore/meta.py'),
                               'fastcore.meta.NewChkMeta': ('meta.html#newchkmeta', 'fastcore/meta.py'),
                               'fastcore.meta.NewChkMeta.__call__': ('meta.html#newchkmeta.__call__', 'fastcore/meta.py'),
                               'fastcore.meta.PrePostInitMeta': ('meta.html#prepostinitmeta', 'fastcore/meta.py'),
                               'fastcore.meta.PrePostInitMeta.__call__': ('meta.html#prepostinitmeta.__call__', 'fastcore/meta.py'),
                               'fastcore.meta._funcs_kwargs': ('meta.html#_funcs_kwargs', 'fastcore/meta.py'),
                               'fastcore.meta._mk_param': ('meta.html#_mk_param', 'fastcore/meta.py'),
                               'fastcore.meta._rm_self': ('meta.html#_rm_self', 'fastcore/meta.py'),
                               'fastcore.meta.anno_dict': ('meta.html#anno_dict', 'fastcore/meta.py'),
                               'fastcore.meta.delegates': ('meta.html#delegates', 'fastcore/meta.py'),
                               'fastcore.meta.empty2none': ('meta.html#empty2none', 'fastcore/meta.py'),
                               'fastcore.meta.funcs_kwargs': ('meta.html#funcs_kwargs', 'fastcore/meta.py'),
                               'fastcore.meta.method': ('meta.html#method', 'fastcore/meta.py'),
                               'fastcore.meta.test_sig': ('meta.html#test_sig', 'fastcore/meta.py'),
                               'fastcore.meta.use_kwargs': ('meta.html#use_kwargs', 'fastcore/meta.py'),
                               'fastcore.meta.use_kwargs_dict': ('meta.html#use_kwargs_dict', 'fastcore/meta.py')},
            'fastcore.nb_imports': {},
            'fastcore.net': { 'fastcore.net.HTTP4xxClientError': ('net.html#http4xxclienterror', 'fastcore/net.py'),
                              'fastcore.net.HTTP5xxServerError': ('net.html#http5xxservererror', 'fastcore/net.py'),
                              'fastcore.net.Request.summary': ('net.html#request.summary', 'fastcore/net.py'),
                              'fastcore.net._socket_det': ('net.html#_socket_det', 'fastcore/net.py'),
                              'fastcore.net.do_request': ('net.html#do_request', 'fastcore/net.py'),
                              'fastcore.net.start_client': ('net.html#start_client', 'fastcore/net.py'),
                              'fastcore.net.start_server': ('net.html#start_server', 'fastcore/net.py'),
                              'fastcore.net.urlcheck': ('net.html#urlcheck', 'fastcore/net.py'),
                              'fastcore.net.urlclean': ('net.html#urlclean', 'fastcore/net.py'),
                              'fastcore.net.urldest': ('net.html#urldest', 'fastcore/net.py'),
                              'fastcore.net.urljson': ('net.html#urljson', 'fastcore/net.py'),
                              'fastcore.net.urlopen': ('net.html#urlopen', 'fastcore/net.py'),
                              'fastcore.net.urlopener': ('net.html#urlopener', 'fastcore/net.py'),
                              'fastcore.net.urlquote': ('net.html#urlquote', 'fastcore/net.py'),
                              'fastcore.net.urlread': ('net.html#urlread', 'fastcore/net.py'),
                              'fastcore.net.urlrequest': ('net.html#urlrequest', 'fastcore/net.py'),
                              'fastcore.net.urlretrieve': ('net.html#urlretrieve', 'fastcore/net.py'),
                              'fastcore.net.urlsave': ('net.html#urlsave', 'fastcore/net.py'),
                              'fastcore.net.urlsend': ('net.html#urlsend', 'fastcore/net.py'),
                              'fastcore.net.urlvalid': ('net.html#urlvalid', 'fastcore/net.py'),
                              'fastcore.net.urlwrap': ('net.html#urlwrap', 'fastcore/net.py')},
            'fastcore.parallel': { 'fastcore.parallel.ProcessPoolExecutor': ('parallel.html#processpoolexecutor', 'fastcore/parallel.py'),
                                   'fastcore.parallel.ProcessPoolExecutor.__init__': ( 'parallel.html#processpoolexecutor.__init__',
                                                                                       'fastcore/parallel.py'),
                                   'fastcore.parallel.ProcessPoolExecutor.map': ( 'parallel.html#processpoolexecutor.map',
                                                                                  'fastcore/parallel.py'),
                                   'fastcore.parallel.ThreadPoolExecutor': ('parallel.html#threadpoolexecutor', 'fastcore/parallel.py'),
                                   'fastcore.parallel.ThreadPoolExecutor.__init__': ( 'parallel.html#threadpoolexecutor.__init__',
                                                                                      'fastcore/parallel.py'),
                                   'fastcore.parallel.ThreadPoolExecutor.map': ( 'parallel.html#threadpoolexecutor.map',
                                                                                 'fastcore/parallel.py'),
                                   'fastcore.parallel._call': ('parallel.html#_call', 'fastcore/parallel.py'),
                                   'fastcore.parallel._done_pg': ('parallel.html#_done_pg', 'fastcore/parallel.py'),
                                   'fastcore.parallel._f_pg': ('parallel.html#_f_pg', 'fastcore/parallel.py'),
                                   'fastcore.parallel.add_one': ('parallel.html#add_one', 'fastcore/parallel.py'),
                                   'fastcore.parallel.parallel': ('parallel.html#parallel', 'fastcore/parallel.py'),
                                   'fastcore.parallel.parallel_gen': ('parallel.html#parallel_gen', 'fastcore/parallel.py'),
                                   'fastcore.parallel.parallelable': ('parallel.html#parallelable', 'fastcore/parallel.py'),
                                   'fastcore.parallel.run_procs': ('parallel.html#run_procs', 'fastcore/parallel.py'),
                                   'fastcore.parallel.startthread': ('parallel.html#startthread', 'fastcore/parallel.py'),
                                   'fastcore.parallel.threaded': ('parallel.html#threaded', 'fastcore/parallel.py')},
            'fastcore.script': { 'fastcore.script.Param': ('script.html#param', 'fastcore/script.py'),
                                 'fastcore.script.Param.__init__': ('script.html#param.__init__', 'fastcore/script.py'),
                                 'fastcore.script.Param.__repr__': ('script.html#param.__repr__', 'fastcore/script.py'),
                                 'fastcore.script.Param.kwargs': ('script.html#param.kwargs', 'fastcore/script.py'),
                                 'fastcore.script.Param.pre': ('script.html#param.pre', 'fastcore/script.py'),
                                 'fastcore.script.Param.set_default': ('script.html#param.set_default', 'fastcore/script.py'),
                                 'fastcore.script._HelpFormatter': ('script.html#_helpformatter', 'fastcore/script.py'),
                                 'fastcore.script._HelpFormatter.__init__': ('script.html#_helpformatter.__init__', 'fastcore/script.py'),
                                 'fastcore.script._HelpFormatter._expand_help': ( 'script.html#_helpformatter._expand_help',
                                                                                  'fastcore/script.py'),
                                 'fastcore.script.anno_parser': ('script.html#anno_parser', 'fastcore/script.py'),
                                 'fastcore.script.args_from_prog': ('script.html#args_from_prog', 'fastcore/script.py'),
                                 'fastcore.script.bool_arg': ('script.html#bool_arg', 'fastcore/script.py'),
                                 'fastcore.script.call_parse': ('script.html#call_parse', 'fastcore/script.py'),
                                 'fastcore.script.clean_type_str': ('script.html#clean_type_str', 'fastcore/script.py'),
                                 'fastcore.script.store_false': ('script.html#store_false', 'fastcore/script.py'),
                                 'fastcore.script.store_true': ('script.html#store_true', 'fastcore/script.py')},
            'fastcore.shutil': {},
            'fastcore.style': { 'fastcore.style.Style': ('style.html#style', 'fastcore/style.py'),
                                'fastcore.style.Style.__call__': ('style.html#style.__call__', 'fastcore/style.py'),
                                'fastcore.style.Style.__dir__': ('style.html#style.__dir__', 'fastcore/style.py'),
                                'fastcore.style.Style.__getattr__': ('style.html#style.__getattr__', 'fastcore/style.py'),
                                'fastcore.style.Style.__init__': ('style.html#style.__init__', 'fastcore/style.py'),
                                'fastcore.style.Style.__repr__': ('style.html#style.__repr__', 'fastcore/style.py'),
                                'fastcore.style.StyleCode': ('style.html#stylecode', 'fastcore/style.py'),
                                'fastcore.style.StyleCode.__init__': ('style.html#stylecode.__init__', 'fastcore/style.py'),
                                'fastcore.style.StyleCode.__str__': ('style.html#stylecode.__str__', 'fastcore/style.py'),
                                'fastcore.style._demo': ('style.html#_demo', 'fastcore/style.py'),
                                'fastcore.style._mk_codes': ('style.html#_mk_codes', 'fastcore/style.py'),
                                'fastcore.style._reset_code': ('style.html#_reset_code', 'fastcore/style.py'),
                                'fastcore.style.demo': ('style.html#demo', 'fastcore/style.py')},
            'fastcore.test': { 'fastcore.test.ExceptionExpected': ('test.html#exceptionexpected', 'fastcore/test.py'),
                               'fastcore.test.ExceptionExpected.__enter__': ('test.html#exceptionexpected.__enter__', 'fastcore/test.py'),
                               'fastcore.test.ExceptionExpected.__exit__': ('test.html#exceptionexpected.__exit__', 'fastcore/test.py'),
                               'fastcore.test.ExceptionExpected.__init__': ('test.html#exceptionexpected.__init__', 'fastcore/test.py'),
                               'fastcore.test.is_close': ('test.html#is_close', 'fastcore/test.py'),
                               'fastcore.test.nequals': ('test.html#nequals', 'fastcore/test.py'),
                               'fastcore.test.test': ('test.html#test', 'fastcore/test.py'),
                               'fastcore.test.test_close': ('test.html#test_close', 'fastcore/test.py'),
                               'fastcore.test.test_eq': ('test.html#test_eq', 'fastcore/test.py'),
                               'fastcore.test.test_eq_type': ('test.html#test_eq_type', 'fastcore/test.py'),
                               'fastcore.test.test_fail': ('test.html#test_fail', 'fastcore/test.py'),
                               'fastcore.test.test_fig_exists': ('test.html#test_fig_exists', 'fastcore/test.py'),
                               'fastcore.test.test_is': ('test.html#test_is', 'fastcore/test.py'),
                               'fastcore.test.test_ne': ('test.html#test_ne', 'fastcore/test.py'),
                               'fastcore.test.test_shuffled': ('test.html#test_shuffled', 'fastcore/test.py'),
                               'fastcore.test.test_stdout': ('test.html#test_stdout', 'fastcore/test.py'),
                               'fastcore.test.test_warns': ('test.html#test_warns', 'fastcore/test.py')},
            'fastcore.transform': { 'fastcore.transform.DisplayedTransform': ('transform.html#displayedtransform', 'fastcore/transform.py'),
                                    'fastcore.transform.DisplayedTransform.name': ( 'transform.html#displayedtransform.name',
                                                                                    'fastcore/transform.py'),
                                    'fastcore.transform.Func': ('transform.html#func', 'fastcore/transform.py'),
                                    'fastcore.transform.Func.__call__': ('transform.html#func.__call__', 'fastcore/transform.py'),
                                    'fastcore.transform.Func.__init__': ('transform.html#func.__init__', 'fastcore/transform.py'),
                                    'fastcore.transform.Func.__repr__': ('transform.html#func.__repr__', 'fastcore/transform.py'),
                                    'fastcore.transform.Func._get': ('transform.html#func._get', 'fastcore/transform.py'),
                                    'fastcore.transform.InplaceTransform': ('transform.html#inplacetransform', 'fastcore/transform.py'),
                                    'fastcore.transform.InplaceTransform._call': ( 'transform.html#inplacetransform._call',
                                                                                   'fastcore/transform.py'),
                                    'fastcore.transform.ItemTransform': ('transform.html#itemtransform', 'fastcore/transform.py'),
                                    'fastcore.transform.ItemTransform.__call__': ( 'transform.html#itemtransform.__call__',
                                                                                   'fastcore/transform.py'),
                                    'fastcore.transform.ItemTransform._call1': ( 'transform.html#itemtransform._call1',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform.ItemTransform.decode': ( 'transform.html#itemtransform.decode',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline': ('transform.html#pipeline', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__call__': ('transform.html#pipeline.__call__', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__dir__': ('transform.html#pipeline.__dir__', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__getattr__': ( 'transform.html#pipeline.__getattr__',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__getitem__': ( 'transform.html#pipeline.__getitem__',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__init__': ('transform.html#pipeline.__init__', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__repr__': ('transform.html#pipeline.__repr__', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.__setstate__': ( 'transform.html#pipeline.__setstate__',
                                                                                  'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline._is_showable': ( 'transform.html#pipeline._is_showable',
                                                                                  'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.add': ('transform.html#pipeline.add', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.decode': ('transform.html#pipeline.decode', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.setup': ('transform.html#pipeline.setup', 'fastcore/transform.py'),
                                    'fastcore.transform.Pipeline.show': ('transform.html#pipeline.show', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform': ('transform.html#transform', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.__call__': ('transform.html#transform.__call__', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.__init__': ('transform.html#transform.__init__', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.__repr__': ('transform.html#transform.__repr__', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform._call': ('transform.html#transform._call', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform._do_call': ('transform.html#transform._do_call', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.decode': ('transform.html#transform.decode', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.name': ('transform.html#transform.name', 'fastcore/transform.py'),
                                    'fastcore.transform.Transform.setup': ('transform.html#transform.setup', 'fastcore/transform.py'),
                                    'fastcore.transform._Sig': ('transform.html#_sig', 'fastcore/transform.py'),
                                    'fastcore.transform._Sig.__getattr__': ('transform.html#_sig.__getattr__', 'fastcore/transform.py'),
                                    'fastcore.transform._TfmDict': ('transform.html#_tfmdict', 'fastcore/transform.py'),
                                    'fastcore.transform._TfmDict.__setitem__': ( 'transform.html#_tfmdict.__setitem__',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform._TfmMeta': ('transform.html#_tfmmeta', 'fastcore/transform.py'),
                                    'fastcore.transform._TfmMeta.__call__': ('transform.html#_tfmmeta.__call__', 'fastcore/transform.py'),
                                    'fastcore.transform._TfmMeta.__new__': ('transform.html#_tfmmeta.__new__', 'fastcore/transform.py'),
                                    'fastcore.transform._TfmMeta.__prepare__': ( 'transform.html#_tfmmeta.__prepare__',
                                                                                 'fastcore/transform.py'),
                                    'fastcore.transform._get_name': ('transform.html#_get_name', 'fastcore/transform.py'),
                                    'fastcore.transform._is_tfm_method': ('transform.html#_is_tfm_method', 'fastcore/transform.py'),
                                    'fastcore.transform._is_tuple': ('transform.html#_is_tuple', 'fastcore/transform.py'),
                                    'fastcore.transform.compose_tfms': ('transform.html#compose_tfms', 'fastcore/transform.py'),
                                    'fastcore.transform.gather_attr_names': ('transform.html#gather_attr_names', 'fastcore/transform.py'),
                                    'fastcore.transform.gather_attrs': ('transform.html#gather_attrs', 'fastcore/transform.py'),
                                    'fastcore.transform.get_func': ('transform.html#get_func', 'fastcore/transform.py'),
                                    'fastcore.transform.mk_transform': ('transform.html#mk_transform', 'fastcore/transform.py')},
            'fastcore.utils': {},
            'fastcore.xdg': { 'fastcore.xdg._path_from_env': ('xdg.html#_path_from_env', 'fastcore/xdg.py'),
                              'fastcore.xdg._paths_from_env': ('xdg.html#_paths_from_env', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_cache_home': ('xdg.html#xdg_cache_home', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_config_dirs': ('xdg.html#xdg_config_dirs', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_config_home': ('xdg.html#xdg_config_home', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_data_dirs': ('xdg.html#xdg_data_dirs', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_data_home': ('xdg.html#xdg_data_home', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_runtime_dir': ('xdg.html#xdg_runtime_dir', 'fastcore/xdg.py'),
                              'fastcore.xdg.xdg_state_home': ('xdg.html#xdg_state_home', 'fastcore/xdg.py')},
            'fastcore.xtras': { 'fastcore.xtras.ContextManagers': ('xtras.html#contextmanagers', 'fastcore/xtras.py'),
                                'fastcore.xtras.ContextManagers.__enter__': ('xtras.html#contextmanagers.__enter__', 'fastcore/xtras.py'),
                                'fastcore.xtras.ContextManagers.__exit__': ('xtras.html#contextmanagers.__exit__', 'fastcore/xtras.py'),
                                'fastcore.xtras.ContextManagers.__init__': ('xtras.html#contextmanagers.__init__', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer': ('xtras.html#eventtimer', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer.__init__': ('xtras.html#eventtimer.__init__', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer._reset': ('xtras.html#eventtimer._reset', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer.add': ('xtras.html#eventtimer.add', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer.duration': ('xtras.html#eventtimer.duration', 'fastcore/xtras.py'),
                                'fastcore.xtras.EventTimer.freq': ('xtras.html#eventtimer.freq', 'fastcore/xtras.py'),
                                'fastcore.xtras.IterLen': ('xtras.html#iterlen', 'fastcore/xtras.py'),
                                'fastcore.xtras.IterLen.__iter__': ('xtras.html#iterlen.__iter__', 'fastcore/xtras.py'),
                                'fastcore.xtras.PartialFormatter': ('xtras.html#partialformatter', 'fastcore/xtras.py'),
                                'fastcore.xtras.PartialFormatter.__init__': ('xtras.html#partialformatter.__init__', 'fastcore/xtras.py'),
                                'fastcore.xtras.PartialFormatter.check_unused_args': ( 'xtras.html#partialformatter.check_unused_args',
                                                                                       'fastcore/xtras.py'),
                                'fastcore.xtras.PartialFormatter.get_field': ('xtras.html#partialformatter.get_field', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.__repr__': ('xtras.html#path.__repr__', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.delete': ('xtras.html#path.delete', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.ls': ('xtras.html#path.ls', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.mk_write': ('xtras.html#path.mk_write', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.read_json': ('xtras.html#path.read_json', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.readlines': ('xtras.html#path.readlines', 'fastcore/xtras.py'),
                                'fastcore.xtras.Path.relpath': ('xtras.html#path.relpath', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection': ('xtras.html#reindexcollection', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.__getitem__': ( 'xtras.html#reindexcollection.__getitem__',
                                                                                  'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.__getstate__': ( 'xtras.html#reindexcollection.__getstate__',
                                                                                   'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.__init__': ('xtras.html#reindexcollection.__init__', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.__len__': ('xtras.html#reindexcollection.__len__', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.__setstate__': ( 'xtras.html#reindexcollection.__setstate__',
                                                                                   'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection._get': ('xtras.html#reindexcollection._get', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.cache_clear': ( 'xtras.html#reindexcollection.cache_clear',
                                                                                  'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.reindex': ('xtras.html#reindexcollection.reindex', 'fastcore/xtras.py'),
                                'fastcore.xtras.ReindexCollection.shuffle': ('xtras.html#reindexcollection.shuffle', 'fastcore/xtras.py'),
                                'fastcore.xtras._ceil': ('xtras.html#_ceil', 'fastcore/xtras.py'),
                                'fastcore.xtras._has_property_getter': ('xtras.html#_has_property_getter', 'fastcore/xtras.py'),
                                'fastcore.xtras._is_property': ('xtras.html#_is_property', 'fastcore/xtras.py'),
                                'fastcore.xtras._is_type_dispatch': ('xtras.html#_is_type_dispatch', 'fastcore/xtras.py'),
                                'fastcore.xtras._property_getter': ('xtras.html#_property_getter', 'fastcore/xtras.py'),
                                'fastcore.xtras._repr_dict': ('xtras.html#_repr_dict', 'fastcore/xtras.py'),
                                'fastcore.xtras._sparkchar': ('xtras.html#_sparkchar', 'fastcore/xtras.py'),
                                'fastcore.xtras._unpack': ('xtras.html#_unpack', 'fastcore/xtras.py'),
                                'fastcore.xtras._unwrapped_func': ('xtras.html#_unwrapped_func', 'fastcore/xtras.py'),
                                'fastcore.xtras._unwrapped_type_dispatch_func': ( 'xtras.html#_unwrapped_type_dispatch_func',
                                                                                  'fastcore/xtras.py'),
                                'fastcore.xtras.autostart': ('xtras.html#autostart', 'fastcore/xtras.py'),
                                'fastcore.xtras.bunzip': ('xtras.html#bunzip', 'fastcore/xtras.py'),
                                'fastcore.xtras.console_help': ('xtras.html#console_help', 'fastcore/xtras.py'),
                                'fastcore.xtras.dict2obj': ('xtras.html#dict2obj', 'fastcore/xtras.py'),
                                'fastcore.xtras.dumps': ('xtras.html#dumps', 'fastcore/xtras.py'),
                                'fastcore.xtras.get_source_link': ('xtras.html#get_source_link', 'fastcore/xtras.py'),
                                'fastcore.xtras.globtastic': ('xtras.html#globtastic', 'fastcore/xtras.py'),
                                'fastcore.xtras.image_size': ('xtras.html#image_size', 'fastcore/xtras.py'),
                                'fastcore.xtras.is_listy': ('xtras.html#is_listy', 'fastcore/xtras.py'),
                                'fastcore.xtras.join_path_file': ('xtras.html#join_path_file', 'fastcore/xtras.py'),
                                'fastcore.xtras.load_pickle': ('xtras.html#load_pickle', 'fastcore/xtras.py'),
                                'fastcore.xtras.loads': ('xtras.html#loads', 'fastcore/xtras.py'),
                                'fastcore.xtras.loads_multi': ('xtras.html#loads_multi', 'fastcore/xtras.py'),
                                'fastcore.xtras.local2utc': ('xtras.html#local2utc', 'fastcore/xtras.py'),
                                'fastcore.xtras.mapped': ('xtras.html#mapped', 'fastcore/xtras.py'),
                                'fastcore.xtras.maybe_open': ('xtras.html#maybe_open', 'fastcore/xtras.py'),
                                'fastcore.xtras.mkdir': ('xtras.html#mkdir', 'fastcore/xtras.py'),
                                'fastcore.xtras.modified_env': ('xtras.html#modified_env', 'fastcore/xtras.py'),
                                'fastcore.xtras.modify_exception': ('xtras.html#modify_exception', 'fastcore/xtras.py'),
                                'fastcore.xtras.obj2dict': ('xtras.html#obj2dict', 'fastcore/xtras.py'),
                                'fastcore.xtras.open_file': ('xtras.html#open_file', 'fastcore/xtras.py'),
                                'fastcore.xtras.partial_format': ('xtras.html#partial_format', 'fastcore/xtras.py'),
                                'fastcore.xtras.repo_details': ('xtras.html#repo_details', 'fastcore/xtras.py'),
                                'fastcore.xtras.repr_dict': ('xtras.html#repr_dict', 'fastcore/xtras.py'),
                                'fastcore.xtras.round_multiple': ('xtras.html#round_multiple', 'fastcore/xtras.py'),
                                'fastcore.xtras.run': ('xtras.html#run', 'fastcore/xtras.py'),
                                'fastcore.xtras.save_pickle': ('xtras.html#save_pickle', 'fastcore/xtras.py'),
                                'fastcore.xtras.set_num_threads': ('xtras.html#set_num_threads', 'fastcore/xtras.py'),
                                'fastcore.xtras.shufflish': ('xtras.html#shufflish', 'fastcore/xtras.py'),
                                'fastcore.xtras.sparkline': ('xtras.html#sparkline', 'fastcore/xtras.py'),
                                'fastcore.xtras.stringfmt_names': ('xtras.html#stringfmt_names', 'fastcore/xtras.py'),
                                'fastcore.xtras.trace': ('xtras.html#trace', 'fastcore/xtras.py'),
                                'fastcore.xtras.truncstr': ('xtras.html#truncstr', 'fastcore/xtras.py'),
                                'fastcore.xtras.untar_dir': ('xtras.html#untar_dir', 'fastcore/xtras.py'),
                                'fastcore.xtras.utc2local': ('xtras.html#utc2local', 'fastcore/xtras.py'),
                                'fastcore.xtras.walk': ('xtras.html#walk', 'fastcore/xtras.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_meta.ipynb.

# %% auto 0
__all__ = ['test_sig', 'FixSigMeta', 'PrePostInitMeta', 'AutoInit', 'NewChkMeta', 'BypassNewMeta', 'empty2none', 'anno_dict',
           'use_kwargs_dict', 'use_kwargs', 'delegates', 'method', 'funcs_kwargs']

# %% ../nbs/07_meta.ipynb 1
from .imports import *
from .test import *
from contextlib import contextmanager
from copy import copy
import inspect

# %% ../nbs/07_meta.ipynb 5
def test_sig(f, b):
    "Test the signature of an object"
    test_eq(str(inspect.signature(f)), b)

# %% ../nbs/07_meta.ipynb 7
def _rm_self(sig):
    sigd = dict(sig.parameters)
    sigd.pop('self')
    return sig.replace(parameters=sigd.values())

# %% ../nbs/07_meta.ipynb 8
class FixSigMeta(type):
    "A metaclass that fixes the signature on classes that override `__new__`"
    def __new__(cls, name, bases, dict):
        res = super().__new__(cls, name, bases, dict)
        if res.__init__ is not object.__init__: res.__signature__ = _rm_self(inspect.signature(res.__init__))
        return res

# %% ../nbs/07_meta.ipynb 23
class PrePostInitMeta(FixSigMeta):
    "A metaclass that calls optional `__pre_init__` and `__post_init__` methods"
    def __call__(cls, *args, **kwargs):
        res = cls.__new__(cls)
        if type(res)==cls:
            if hasattr(res,'__pre_init__'): res.__pre_init__(*args,**kwargs)
            res.__init__(*args,**kwargs)
            if hasattr(res,'__post_init__'): res.__post_init__(*args,**kwargs)
        return res

# %% ../nbs/07_meta.ipynb 28
class AutoInit(metaclass=PrePostInitMeta):
    "Same as `object`, but no need for subclasses to call `super().__init__`"
    def __pre_init__(self, *args, **kwargs): super().__init__(*args, **kwargs)

# %% ../nbs/07_meta.ipynb 31
class NewChkMeta(FixSigMeta):
    "Metaclass to avoid recreating object passed to constructor"
    def __call__(cls, x=None, *args, **kwargs):
        if not args and not kwargs and x is not None and isinstance(x,cls): return x
        res = super().__call__(*((x,) + args), **kwargs)
        return res

# %% ../nbs/07_meta.ipynb 44
class BypassNewMeta(FixSigMeta):
    "Metaclass: casts `x` to this class if it's of type `cls._bypass_type`"
    def __call__(cls, x=None, *args, **kwargs):
        if hasattr(cls, '_new_meta'): x = cls._new_meta(x, *args, **kwargs)
        elif not isinstance(x,getattr(cls,'_bypass_type',object)) or len(args) or len(kwargs):
            x = super().__call__(*((x,)+args), **kwargs)
        if cls!=x.__class__: x.__class__ = cls
        return x

# %% ../nbs/07_meta.ipynb 54
def empty2none(p):
    "Replace `Parameter.empty` with `None`"
    return None if p==inspect.Parameter.empty else p

# %% ../nbs/07_meta.ipynb 55
def anno_dict(f):
    "`__annotation__ dictionary with `empty` cast to `None`, returning empty if doesn't exist"
    return {k:empty2none(v) for k,v in getattr(f, '__annotations__', {}).items()}

# %% ../nbs/07_meta.ipynb 57
def _mk_param(n,d=None): return inspect.Parameter(n, inspect.Parameter.KEYWORD_ONLY, default=d)

# %% ../nbs/07_meta.ipynb 58
def use_kwargs_dict(keep=False, **kwargs):
    "Decorator: replace `**kwargs` in signature with `names` params"
    def _f(f):
        sig = inspect.signature(f)
        sigd = dict(sig.parameters)
        k = sigd.pop('kwargs')
        s2 = {n:_mk_param(n,d) for n,d in kwargs.items() if n not in sigd}
        sigd.update(s2)
        if keep: sigd['kwargs'] = k
        f.__signature__ = sig.replace(parameters=sigd.values())
        return f
    return _f

# %% ../nbs/07_meta.ipynb 63
def use_kwargs(names, keep=False):
    "Decorator: replace `**kwargs` in signature with `names` params"
    def _f(f):
        sig = inspect.signature(f)
        sigd = dict(sig.parameters)
        k = sigd.pop('kwargs')
        s2 = {n:_mk_param(n) for n in names if n not in sigd}
        sigd.update(s2)
        if keep: sigd['kwargs'] = k
        f.__signature__ = sig.replace(parameters=sigd.values())
        return f
    return _f

# %% ../nbs/07_meta.ipynb 68
def delegates(to:FunctionType=None, # Delegatee
              keep=False, # Keep `kwargs` in decorated function?
              but:list=None): # Exclude these parameters from signature
    "Decorator: replace `**kwargs` in signature with params from `to`"
    if but is None: but = []
    def _f(f):
        if to is None: to_f,from_f = f.__base__.__init__,f.__init__
        else:          to_f,from_f = to.__init__ if isinstance(to,type) else to,f
        from_f = getattr(from_f,'__func__',from_f)
        to_f = getattr(to_f,'__func__',to_f)
        if hasattr(from_f,'__delwrap__'): return f
        sig = inspect.signature(from_f)
        sigd = dict(sig.parameters)
        k = sigd.pop('kwargs')
        s2 = {k:v.replace(kind=inspect.Parameter.KEYWORD_ONLY) for k,v in inspect.signature(to_f).parameters.items()
              if v.default != inspect.Parameter.empty and k not in sigd and k not in but}
        anno = {k:v for k,v in getattr(to_f, "__annotations__", {}).items() if k not in sigd and k not in but}
        sigd.update(s2)
        if keep: sigd['kwargs'] = k
        else: from_f.__delwrap__ = to_f
        from_f.__signature__ = sig.replace(parameters=sigd.values())
        if hasattr(from_f, '__annotations__'): from_f.__annotations__.update(anno)
        return f
    return _f

# %% ../nbs/07_meta.ipynb 87
def method(f):
    "Mark `f` as a method"
    # `1` is a dummy instance since Py3 doesn't allow `None` any more
    return MethodType(f, 1)

# %% ../nbs/07_meta.ipynb 90
def _funcs_kwargs(cls, as_method):
    old_init = cls.__init__
    def _init(self, *args, **kwargs):
        for k in cls._methods:
            arg = kwargs.pop(k,None)
            if arg is not None:
                if as_method: arg = method(arg)
                if isinstance(arg,MethodType): arg = MethodType(arg.__func__, self)
                setattr(self, k, arg)
        old_init(self, *args, **kwargs)
    functools.update_wrapper(_init, old_init)
    cls.__init__ = use_kwargs(cls._methods)(_init)
    if hasattr(cls, '__signature__'): cls.__signature__ = _rm_self(inspect.signature(cls.__init__))
    return cls

# %% ../nbs/07_meta.ipynb 91
def funcs_kwargs(as_method=False):
    "Replace methods in `cls._methods` with those from `kwargs`"
    if callable(as_method): return _funcs_kwargs(as_method, False)
    return partial(_funcs_kwargs, as_method=as_method)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/10_style.ipynb.

# %% auto 0
__all__ = ['style_codes', 'S', 'StyleCode', 'Style', 'demo']

# %% ../nbs/10_style.ipynb 3
# Source: https://misc.flogisoft.com/bash/tip_colors_and_formatting
_base = 'red green yellow blue magenta cyan'
_regular = f'black {_base} light_gray'
_intense = 'dark_gray ' + ' '.join('light_'+o for o in _base.split()) + ' white'
_fmt = 'bold dim italic underline blink <na> invert hidden strikethrough'

# %% ../nbs/10_style.ipynb 4
class StyleCode:
    "An escape sequence for styling terminal text."
    def __init__(self, name, code, typ): self.name,self.code,self.typ = name,code,typ
    def __str__(self): return f'\033[{self.code}m'

# %% ../nbs/10_style.ipynb 7
def _mk_codes(s, start, typ, fmt=None, **kwargs):
    d = {k:i for i,k in enumerate(s.split())} if isinstance(s, str) else s
    res = {k if fmt is None else fmt.format(k):start+v for k,v in d.items()}
    res.update(kwargs)
    return {k:StyleCode(k,v,typ) for k,v in res.items()}

# %% ../nbs/10_style.ipynb 8
# Hardcode `reset_bold=22` since 21 is not always supported
# See: https://gist.github.com/fnky/458719343aabd01cfb17a3a4f7296797
style_codes = {**_mk_codes(_regular, 30,  'fg',                default=39),
               **_mk_codes(_intense, 90,  'fg'),
               **_mk_codes(_regular, 40,  'bg',    '{}_bg',    default_bg=49),
               **_mk_codes(_intense, 100, 'bg',    '{}_bg'),
               **_mk_codes(_fmt,     1,   'fmt'),
               **_mk_codes(_fmt,     21,  'reset', 'reset_{}', reset=0, reset_bold=22)}
style_codes = {k:v for k,v in style_codes.items() if '<na>' not in k}

# %% ../nbs/10_style.ipynb 9
def _reset_code(s):
    if s.typ == 'fg':  return style_codes['default']
    if s.typ == 'bg':  return style_codes['default_bg']
    if s.typ == 'fmt': return style_codes['reset_'+s.name]

# %% ../nbs/10_style.ipynb 10
class Style:
    "A minimal terminal text styler."
    def __init__(self, codes=None): self.codes = [] if codes is None else codes
    def __dir__(self): return style_codes.keys()
    def __getattr__(self, k):
        try: return Style(self.codes+[style_codes[k]])
        except KeyError: return super().__getattr__(k)
    def __call__(self, obj):
        set_ = ''.join(str(o) for o in self.codes)
        reset = ''.join(sorted('' if o is None else str(o) for o in set(_reset_code(o) for o in self.codes)))
        return set_ + str(obj) + reset
    def __repr__(self):
        nm = type(self).__name__
        res = f'<{nm}: '
        res += ' '.join(o.name for o in self.codes) if self.codes else 'none'
        return res+'>'

# %% ../nbs/10_style.ipynb 12
S = Style()

# %% ../nbs/10_style.ipynb 25
def _demo(name, code):
    s = getattr(S,name)
    print(s(f'{code.code:>3}    {name:16}'))

# %% ../nbs/10_style.ipynb 26
def demo():
    "Demonstrate all available styles and their codes."
    for k,v in style_codes.items(): _demo(k,v)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_xtras.ipynb.

# %% ../nbs/03_xtras.ipynb 1
from __future__ import annotations

# %% auto 0
__all__ = ['spark_chars', 'walk', 'globtastic', 'maybe_open', 'mkdir', 'image_size', 'bunzip', 'loads', 'loads_multi', 'dumps',
           'untar_dir', 'repo_details', 'run', 'open_file', 'save_pickle', 'load_pickle', 'dict2obj', 'obj2dict',
           'repr_dict', 'is_listy', 'mapped', 'IterLen', 'ReindexCollection', 'get_source_link', 'truncstr',
           'sparkline', 'modify_exception', 'round_multiple', 'set_num_threads', 'join_path_file', 'autostart',
           'EventTimer', 'stringfmt_names', 'PartialFormatter', 'partial_format', 'utc2local', 'local2utc', 'trace',
           'modified_env', 'ContextManagers', 'shufflish', 'console_help']

# %% ../nbs/03_xtras.ipynb 2
from .imports import *
from .foundation import *
from .basics import *
from importlib import import_module
from functools import wraps
import string,time
from contextlib import contextmanager,ExitStack
from datetime import datetime, timezone

# %% ../nbs/03_xtras.ipynb 7
def walk(
    path:Path|str, # path to start searching
    symlinks:bool=True, # follow symlinks?
    keep_file:callable=ret_true, # function that returns True for wanted files
    keep_folder:callable=ret_true, # function that returns True for folders to enter
    skip_folder:callable=ret_false, # function that returns True for folders to skip
    func:callable=os.path.join, # function to apply to each matched file
    ret_folders:bool=False  # return folders, not just files
):
    "Generator version of `os.walk`, using functions to filter files and folders"
    from copy import copy
    for root,dirs,files in os.walk(path, followlinks=symlinks):
        if keep_folder(root,''):
            if ret_folders: yield func(root, '')
            yield from (func(root, name) for name in files if keep_file(root,name))
        for name in copy(dirs):
            if skip_folder(root,name): dirs.remove(name)

# %% ../nbs/03_xtras.ipynb 8
def globtastic(
    path:Path|str, # path to start searching
    recursive:bool=True, # search subfolders
    symlinks:bool=True, # follow symlinks?
    file_glob:str=None, # Only include files matching glob
    file_re:str=None, # Only include files matching regex
    folder_re:str=None, # Only enter folders matching regex
    skip_file_glob:str=None, # Skip files matching glob
    skip_file_re:str=None, # Skip files matching regex
    skip_folder_re:str=None, # Skip folders matching regex,
    func:callable=os.path.join, # function to apply to each matched file
    ret_folders:bool=False  # return folders, not just files
)->L: # Paths to matched files
    "A more powerful `glob`, including regex matches, symlink handling, and skip parameters"
    from fnmatch import fnmatch
    path = Path(path)
    if path.is_file(): return L([path])
    if not recursive: skip_folder_re='.'
    file_re,folder_re = compile_re(file_re),compile_re(folder_re)
    skip_file_re,skip_folder_re = compile_re(skip_file_re),compile_re(skip_folder_re)
    def _keep_file(root, name):
        return (not file_glob or fnmatch(name, file_glob)) and (
                not file_re or file_re.search(name)) and (
                not skip_file_glob or not fnmatch(name, skip_file_glob)) and (
                not skip_file_re or not skip_file_re.search(name))
    def _keep_folder(root, name): return not folder_re or folder_re.search(os.path.join(root,name))
    def _skip_folder(root, name): return skip_folder_re and skip_folder_re.search(name)
    return L(walk(path, symlinks=symlinks, keep_file=_keep_file, keep_folder=_keep_folder, skip_folder=_skip_folder,
                  func=func, ret_folders=ret_folders))

# %% ../nbs/03_xtras.ipynb 10
@contextmanager
def maybe_open(f, mode='r', **kwargs):
    "Context manager: open `f` if it is a path (and close on exit)"
    if isinstance(f, (str,os.PathLike)):
        with open(f, mode, **kwargs) as f: yield f
    else: yield f

# %% ../nbs/03_xtras.ipynb 26
def mkdir(path, exist_ok=False, parents=False, overwrite=False, **kwargs):
    "Creates and returns a directory defined by `path`, optionally removing previous existing directory if `overwrite` is `True`"
    import shutil
    path = Path(path)
    if path.exists() and overwrite: shutil.rmtree(path)
    path.mkdir(exist_ok=exist_ok, parents=parents, **kwargs)
    return path

# %% ../nbs/03_xtras.ipynb 28
def image_size(fn):
    "Tuple of (w,h) for png, gif, or jpg; `None` otherwise"
    import imghdr,struct
    def _jpg_size(f):
        size,ftype = 2,0
        while not 0xc0 <= ftype <= 0xcf:
            f.seek(size, 1)
            byte = f.read(1)
            while ord(byte) == 0xff: byte = f.read(1)
            ftype = ord(byte)
            size = struct.unpack('>H', f.read(2))[0] - 2
        f.seek(1, 1)  # `precision'
        h,w = struct.unpack('>HH', f.read(4))
        return w,h

    def _gif_size(f): return struct.unpack('<HH', head[6:10])

    def _png_size(f):
        assert struct.unpack('>i', head[4:8])[0]==0x0d0a1a0a
        return struct.unpack('>ii', head[16:24])
    d = dict(png=_png_size, gif=_gif_size, jpeg=_jpg_size)
    with maybe_open(fn, 'rb') as f: return d[imghdr.what(f)](f)

# %% ../nbs/03_xtras.ipynb 30
def bunzip(fn):
    "bunzip `fn`, raising exception if output already exists"
    fn = Path(fn)
    assert fn.exists(), f"{fn} doesn't exist"
    out_fn = fn.with_suffix('')
    assert not out_fn.exists(), f"{out_fn} already exists"
    import bz2
    with bz2.BZ2File(fn, 'rb') as src, out_fn.open('wb') as dst:
        for d in iter(lambda: src.read(1024*1024), b''): dst.write(d)

# %% ../nbs/03_xtras.ipynb 32
def loads(s, **kw):
    "Same as `json.loads`, but handles `None`"
    if not s: return {}
    try: import ujson as json
    except ModuleNotFoundError: import json
    return json.loads(s, **kw)

# %% ../nbs/03_xtras.ipynb 33
def loads_multi(s:str):
    "Generator of >=0 decoded json dicts, possibly with non-json ignored text at start and end"
    import json
    _dec = json.JSONDecoder()
    while s.find('{')>=0:
        s = s[s.find('{'):]
        obj,pos = _dec.raw_decode(s)
        if not pos: raise ValueError(f'no JSON object found at {pos}')
        yield obj
        s = s[pos:]

# %% ../nbs/03_xtras.ipynb 35
def dumps(obj, **kw):
    "Same as `json.dumps`, but uses `ujson` if available"
    try: import ujson as json
    except ModuleNotFoundError: import json
    else: kw['escape_forward_slashes']=False
    return json.dumps(obj, **kw)

# %% ../nbs/03_xtras.ipynb 36
def _unpack(fname, out):
    import shutil
    shutil.unpack_archive(str(fname), str(out))
    ls = out.ls()
    return ls[0] if len(ls) == 1 else out

# %% ../nbs/03_xtras.ipynb 37
def untar_dir(fname, dest, rename=False, overwrite=False):
    "untar `file` into `dest`, creating a directory if the root contains more than one item"
    import tempfile,shutil
    with tempfile.TemporaryDirectory() as d:
        out = Path(d)/remove_suffix(Path(fname).stem, '.tar')
        out.mkdir()
        if rename: dest = dest/out.name
        else:
            src = _unpack(fname, out)
            dest = dest/src.name
        if dest.exists():
            if overwrite: shutil.rmtree(dest) if dest.is_dir() else dest.unlink()
            else: return dest
        if rename: src = _unpack(fname, out)
        shutil.move(str(src), dest)
        return dest

# %% ../nbs/03_xtras.ipynb 45
def repo_details(url):
    "Tuple of `owner,name` from ssh or https git repo `url`"
    res = remove_suffix(url.strip(), '.git')
    res = res.split(':')[-1]
    return res.split('/')[-2:]

# %% ../nbs/03_xtras.ipynb 47
def run(cmd, *rest, same_in_win=False, ignore_ex=False, as_bytes=False, stderr=False):
    "Pass `cmd` (splitting with `shlex` if string) to `subprocess.run`; return `stdout`; raise `IOError` if fails"
    # Even the command is same on Windows, we have to add `cmd /c `"
    import subprocess
    if rest:
        if sys.platform == 'win32' and same_in_win:
            cmd = ('cmd', '/c', cmd, *rest)
        else:
            cmd = (cmd,)+rest
    elif isinstance(cmd, str):
        if sys.platform == 'win32' and same_in_win: cmd = 'cmd /c ' + cmd
        import shlex
        cmd = shlex.split(cmd)
    elif isinstance(cmd, list):
        if sys.platform == 'win32' and same_in_win: cmd = ['cmd', '/c'] + cmd
    res = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout = res.stdout
    if stderr and res.stderr: stdout += b' ;; ' + res.stderr
    if not as_bytes: stdout = stdout.decode().strip()
    if ignore_ex: return (res.returncode, stdout)
    if res.returncode: raise IOError(stdout)
    return stdout

# %% ../nbs/03_xtras.ipynb 55
def open_file(fn, mode='r', **kwargs):
    "Open a file, with optional compression if gz or bz2 suffix"
    if isinstance(fn, io.IOBase): return fn
    import bz2,gzip,zipfile
    fn = Path(fn)
    if   fn.suffix=='.bz2': return bz2.BZ2File(fn, mode, **kwargs)
    elif fn.suffix=='.gz' : return gzip.GzipFile(fn, mode, **kwargs)
    elif fn.suffix=='.zip': return zipfile.ZipFile(fn, mode, **kwargs)
    else: return open(fn,mode, **kwargs)

# %% ../nbs/03_xtras.ipynb 56
def save_pickle(fn, o):
    "Save a pickle file, to a file name or opened file"
    import pickle
    with open_file(fn, 'wb') as f: pickle.dump(o, f)

# %% ../nbs/03_xtras.ipynb 57
def load_pickle(fn):
    "Load a pickle file from a file name or opened file"
    import pickle
    with open_file(fn, 'rb') as f: return pickle.load(f)

# %% ../nbs/03_xtras.ipynb 60
def dict2obj(d, list_func=L, dict_func=AttrDict):
    "Convert (possibly nested) dicts (or lists of dicts) to `AttrDict`"
    if isinstance(d, (L,list)): return list_func(d).map(dict2obj)
    if not isinstance(d, dict): return d
    return dict_func(**{k:dict2obj(v) for k,v in d.items()})

# %% ../nbs/03_xtras.ipynb 65
def obj2dict(d):
    "Convert (possibly nested) AttrDicts (or lists of AttrDicts) to `dict`"
    if isinstance(d, (L,list)): return list(L(d).map(obj2dict))
    if not isinstance(d, dict): return d
    return dict(**{k:obj2dict(v) for k,v in d.items()})

# %% ../nbs/03_xtras.ipynb 68
def _repr_dict(d, lvl):
    if isinstance(d,dict):
        its = [f"{k}: {_repr_dict(v,lvl+1)}" for k,v in d.items()]
    elif isinstance(d,(list,L)): its = [_repr_dict(o,lvl+1) for o in d]
    else: return str(d)
    return '\n' + '\n'.join([" "*(lvl*2) + "- " + o for o in its])

# %% ../nbs/03_xtras.ipynb 69
def repr_dict(d):
    "Print nested dicts and lists, such as returned by `dict2obj`"
    return _repr_dict(d,0).strip()

# %% ../nbs/03_xtras.ipynb 71
def is_listy(x):
    "`isinstance(x, (tuple,list,L,slice,Generator))`"
    return isinstance(x, (tuple,list,L,slice,Generator))

# %% ../nbs/03_xtras.ipynb 73
def mapped(f, it):
    "map `f` over `it`, unless it's not listy, in which case return `f(it)`"
    return L(it).map(f) if is_listy(it) else f(it)

# %% ../nbs/03_xtras.ipynb 77
@patch
def readlines(self:Path, hint=-1, encoding='utf8'):
    "Read the content of `self`"
    with self.open(encoding=encoding) as f: return f.readlines(hint)

# %% ../nbs/03_xtras.ipynb 78
@patch
def read_json(self:Path, encoding=None, errors=None):
    "Same as `read_text` followed by `loads`"
    return loads(self.read_text(encoding=encoding, errors=errors))

# %% ../nbs/03_xtras.ipynb 79
@patch
def mk_write(self:Path, data, encoding=None, errors=None, mode=511):
    "Make all parent dirs of `self`, and write `data`"
    self.parent.mkdir(exist_ok=True, parents=True, mode=mode)
    self.write_text(data, encoding=encoding, errors=errors)

# %% ../nbs/03_xtras.ipynb 80
@patch
def relpath(self:Path, start=None):
    "Same as `os.path.relpath`, but returns a `Path`, and resolves symlinks"
    return Path(os.path.relpath(self.resolve(), Path(start).resolve()))

# %% ../nbs/03_xtras.ipynb 83
@patch
def ls(self:Path, n_max=None, file_type=None, file_exts=None):
    "Contents of path as a list"
    import mimetypes
    extns=L(file_exts)
    if file_type: extns += L(k for k,v in mimetypes.types_map.items() if v.startswith(file_type+'/'))
    has_extns = len(extns)==0
    res = (o for o in self.iterdir() if has_extns or o.suffix in extns)
    if n_max is not None: res = itertools.islice(res, n_max)
    return L(res)

# %% ../nbs/03_xtras.ipynb 89
@patch
def __repr__(self:Path):
    b = getattr(Path, 'BASE_PATH', None)
    if b:
        try: self = self.relative_to(b)
        except: pass
    return f"Path({self.as_posix()!r})"

# %% ../nbs/03_xtras.ipynb 92
@patch
def delete(self:Path):
    "Delete a file, symlink, or directory tree"
    if not self.exists(): return
    if self.is_dir():
        import shutil
        shutil.rmtree(self)
    else: self.unlink()

# %% ../nbs/03_xtras.ipynb 94
class IterLen:
    "Base class to add iteration to anything supporting `__len__` and `__getitem__`"
    def __iter__(self): return (self[i] for i in range_of(self))

# %% ../nbs/03_xtras.ipynb 95
@docs
class ReindexCollection(GetAttr, IterLen):
    "Reindexes collection `coll` with indices `idxs` and optional LRU cache of size `cache`"
    _default='coll'
    def __init__(self, coll, idxs=None, cache=None, tfm=noop):
        if idxs is None: idxs = L.range(coll)
        store_attr()
        if cache is not None: self._get = functools.lru_cache(maxsize=cache)(self._get)

    def _get(self, i): return self.tfm(self.coll[i])
    def __getitem__(self, i): return self._get(self.idxs[i])
    def __len__(self): return len(self.coll)
    def reindex(self, idxs): self.idxs = idxs
    def shuffle(self):
        import random
        random.shuffle(self.idxs)
    def cache_clear(self): self._get.cache_clear()
    def __getstate__(self): return {'coll': self.coll, 'idxs': self.idxs, 'cache': self.cache, 'tfm': self.tfm}
    def __setstate__(self, s): self.coll,self.idxs,self.cache,self.tfm = s['coll'],s['idxs'],s['cache'],s['tfm']

    _docs = dict(reindex="Replace `self.idxs` with idxs",
                shuffle="Randomly shuffle indices",
                cache_clear="Clear LRU cache")

# %% ../nbs/03_xtras.ipynb 114
def _is_type_dispatch(x): return type(x).__name__ == "TypeDispatch"
def _unwrapped_type_dispatch_func(x): return x.first() if _is_type_dispatch(x) else x

def _is_property(x): return type(x)==property
def _has_property_getter(x): return _is_property(x) and hasattr(x, 'fget') and hasattr(x.fget, 'func')
def _property_getter(x): return x.fget.func if _has_property_getter(x) else x

def _unwrapped_func(x):
    x = _unwrapped_type_dispatch_func(x)
    x = _property_getter(x)
    return x


def get_source_link(func):
    "Return link to `func` in source code"
    import inspect
    func = _unwrapped_func(func)
    try: line = inspect.getsourcelines(func)[1]
    except Exception: return ''
    mod = inspect.getmodule(func)
    module = mod.__name__.replace('.', '/') + '.py'
    try:
        nbdev_mod = import_module(mod.__package__.split('.')[0] + '._nbdev')
        return f"{nbdev_mod.git_url}{module}#L{line}"
    except: return f"{module}#L{line}"

# %% ../nbs/03_xtras.ipynb 118
def truncstr(s:str, maxlen:int, suf:str='…', space='')->str:
    "Truncate `s` to length `maxlen`, adding suffix `suf` if truncated"
    return s[:maxlen-len(suf)]+suf if len(s)+len(space)>maxlen else s+space

# %% ../nbs/03_xtras.ipynb 120
spark_chars = '▁▂▃▅▆▇'

# %% ../nbs/03_xtras.ipynb 121
def _ceil(x, lim=None): return x if (not lim or x <= lim) else lim

def _sparkchar(x, mn, mx, incr, empty_zero):
    if x is None or (empty_zero and not x): return ' '
    if incr == 0: return spark_chars[0]
    res = int((_ceil(x,mx)-mn)/incr-0.5)
    return spark_chars[res]

# %% ../nbs/03_xtras.ipynb 122
def sparkline(data, mn=None, mx=None, empty_zero=False):
    "Sparkline for `data`, with `None`s (and zero, if `empty_zero`) shown as empty column"
    valid = [o for o in data if o is not None]
    if not valid: return ' '
    mn,mx,n = ifnone(mn,min(valid)),ifnone(mx,max(valid)),len(spark_chars)
    res = [_sparkchar(x=o, mn=mn, mx=mx, incr=(mx-mn)/n, empty_zero=empty_zero) for o in data]
    return ''.join(res)

# %% ../nbs/03_xtras.ipynb 126
def modify_exception(
    e:Exception, # An exception
    msg:str=None, # A custom message
    replace:bool=False, # Whether to replace e.args with [msg]
) -> Exception:
    "Modifies `e` with a custom message attached"
    e.args = [f'{e.args[0]} {msg}'] if not replace and len(e.args) > 0 else [msg]
    return e

# %% ../nbs/03_xtras.ipynb 128
def round_multiple(x, mult, round_down=False):
    "Round `x` to nearest multiple of `mult`"
    def _f(x_): return (int if round_down else round)(x_/mult)*mult
    res = L(x).map(_f)
    return res if is_listy(x) else res[0]

# %% ../nbs/03_xtras.ipynb 130
def set_num_threads(nt):
    "Get numpy (and others) to use `nt` threads"
    try: import mkl; mkl.set_num_threads(nt)
    except: pass
    try: import torch; torch.set_num_threads(nt)
    except: pass
    os.environ['IPC_ENABLE']='1'
    for o in ['OPENBLAS_NUM_THREADS','NUMEXPR_NUM_THREADS','OMP_NUM_THREADS','MKL_NUM_THREADS']:
        os.environ[o] = str(nt)

# %% ../nbs/03_xtras.ipynb 132
def join_path_file(file, path, ext=''):
    "Return `path/file` if file is a string or a `Path`, file otherwise"
    if not isinstance(file, (str, Path)): return file
    path.mkdir(parents=True, exist_ok=True)
    return path/f'{file}{ext}'

# %% ../nbs/03_xtras.ipynb 135
def autostart(g):
    "Decorator that automatically starts a generator"
    @functools.wraps(g)
    def f():
        r = g()
        next(r)
        return r
    return f

# %% ../nbs/03_xtras.ipynb 136
class EventTimer:
    "An event timer with history of `store` items of time `span`"

    def __init__(self, store=5, span=60):
        import collections
        self.hist,self.span,self.last = collections.deque(maxlen=store),span,time.perf_counter()
        self._reset()

    def _reset(self): self.start,self.events = self.last,0

    def add(self, n=1):
        "Record `n` events"
        if self.duration>self.span:
            self.hist.append(self.freq)
            self._reset()
        self.events +=n
        self.last = time.perf_counter()

    @property
    def duration(self): return time.perf_counter()-self.start
    @property
    def freq(self): return self.events/self.duration

# %% ../nbs/03_xtras.ipynb 140
_fmt = string.Formatter()

# %% ../nbs/03_xtras.ipynb 141
def stringfmt_names(s:str)->list:
    "Unique brace-delimited names in `s`"
    return uniqueify(o[1] for o in _fmt.parse(s) if o[1])

# %% ../nbs/03_xtras.ipynb 143
class PartialFormatter(string.Formatter):
    "A `string.Formatter` that doesn't error on missing fields, and tracks missing fields and unused args"
    def __init__(self):
        self.missing = set()
        super().__init__()

    def get_field(self, nm, args, kwargs):
        try: return super().get_field(nm, args, kwargs)
        except KeyError:
            self.missing.add(nm)
            return '{'+nm+'}',nm

    def check_unused_args(self, used, args, kwargs):
        self.xtra = filter_keys(kwargs, lambda o: o not in used)

# %% ../nbs/03_xtras.ipynb 145
def partial_format(s:str, **kwargs):
    "string format `s`, ignoring missing field errors, returning missing and extra fields"
    fmt = PartialFormatter()
    res = fmt.format(s, **kwargs)
    return res,list(fmt.missing),fmt.xtra

# %% ../nbs/03_xtras.ipynb 148
def utc2local(dt:datetime)->datetime:
    "Convert `dt` from UTC to local time"
    return dt.replace(tzinfo=timezone.utc).astimezone(tz=None)

# %% ../nbs/03_xtras.ipynb 150
def local2utc(dt:datetime)->datetime:
    "Convert `dt` from local to UTC time"
    return dt.replace(tzinfo=None).astimezone(tz=timezone.utc)

# %% ../nbs/03_xtras.ipynb 152
def trace(f):
    "Add `set_trace` to an existing function `f`"
    from pdb import set_trace
    if getattr(f, '_traced', False): return f
    def _inner(*args,**kwargs):
        set_trace()
        return f(*args,**kwargs)
    _inner._traced = True
    return _inner

# %% ../nbs/03_xtras.ipynb 154
@contextmanager
def modified_env(*delete, **replace):
    "Context manager temporarily modifying `os.environ` by deleting `delete` and replacing `replace`"
    prev = dict(os.environ)
    try:
        os.environ.update(replace)
        for k in delete: os.environ.pop(k, None)
        yield
    finally:
        os.environ.clear()
        os.environ.update(prev)

# %% ../nbs/03_xtras.ipynb 156
class ContextManagers(GetAttr):
    "Wrapper for `contextlib.ExitStack` which enters a collection of context managers"
    def __init__(self, mgrs): self.default,self.stack = L(mgrs),ExitStack()
    def __enter__(self): self.default.map(self.stack.enter_context)
    def __exit__(self, *args, **kwargs): self.stack.__exit__(*args, **kwargs)

# %% ../nbs/03_xtras.ipynb 158
def shufflish(x, pct=0.04):
    "Randomly relocate items of `x` up to `pct` of `len(x)` from their starting location"
    n = len(x)
    import random
    return L(x[i] for i in sorted(range_of(x), key=lambda o: o+n*(1+random.random()*pct)))

# %% ../nbs/03_xtras.ipynb 159
def console_help(
    libname:str):  # name of library for console script listing
    "Show help for all console scripts from `libname`"
    from fastcore.style import S
    from pkg_resources import iter_entry_points as ep
    for e in ep('console_scripts'): 
        if e.module_name == libname or e.module_name.startswith(libname+'.'): 
            nm = S.bold.light_blue(e.name)
            print(f'{nm:45}{e.load().__doc__}')

